@@ -28,7 +28,6 @@
 
 #include <iostream>
 
-#include <gnuradio/blocks/multiply_const_ff.h>
 #include <gnuradio/prefs.h>
 #include <gnuradio/top_block.h>
 #include <osmosdr/source.h>
@@ -50,9 +49,11 @@
 #include <gnuradio/audio/sink.h>
 #endif
 
+#define DEFAULT_AUDIO_GAIN -6.0
+#define TARGET_QUAD_RATE 1e6
 
 /**
- * @brief Public contructor.
+ * @brief Public constructor.
  * @param input_device Input device specifier.
  * @param audio_device Audio output device specifier,
  *                     e.g. hw:0 when using ALSA or Portaudio.
@@ -103,35 +104,26 @@ receiver::receiver(const std::string input_device,
             d_decim = 1;
         }
 
-        d_quad_rate = d_input_rate / (double)d_decim;
+        d_decim_rate = d_input_rate / (double)d_decim;
     }
     else
     {
-        d_quad_rate = d_input_rate;
+        d_decim_rate = d_input_rate;
     }
 
-
-    // create I/Q sink and close it
-    iq_sink = gr::blocks::file_sink::make(sizeof(gr_complex), get_null_file().c_str(), true);
-    iq_sink->set_unbuffered(true);
-    iq_sink->close();
-
-    rx = make_nbrx(d_quad_rate, d_audio_rate);
-    lo = gr::analog::sig_source_c::make(d_quad_rate, gr::analog::GR_SIN_WAVE,
-                                        0.0, 1.0);
-    mixer = gr::blocks::multiply_cc::make();
+    d_ddc_decim = std::max(1, (int)(d_decim_rate / TARGET_QUAD_RATE));
+    d_quad_rate = d_decim_rate / d_ddc_decim;
+    ddc = make_downconverter_cc(d_ddc_decim, 0.0, d_decim_rate);
+    rx  = make_nbrx(d_quad_rate, d_audio_rate);
 
     iq_swap = make_iq_swap_cc(false);
-    dc_corr = make_dc_corr_cc(d_quad_rate, 1.0);
-    iq_fft = make_rx_fft_c(8192u, 0);
+    dc_corr = make_dc_corr_cc(d_decim_rate, 1.0);
+    iq_fft = make_rx_fft_c(8192u, d_decim_rate, gr::filter::firdes::WIN_HANN);
 
-    audio_fft = make_rx_fft_f(8192u);
-    audio_gain0 = gr::blocks::multiply_const_ff::make(0.1);
-    audio_gain1 = gr::blocks::multiply_const_ff::make(0.1);
-
-    wav_sink = gr::blocks::wavfile_sink::make(get_null_file().c_str(), 2,
-                                              (unsigned int) d_audio_rate,
-                                              16);
+    audio_fft = make_rx_fft_f(8192u, gr::filter::firdes::WIN_HANN);
+    audio_gain0 = gr::blocks::multiply_const_ff::make(0);
+    audio_gain1 = gr::blocks::multiply_const_ff::make(0);
+    set_af_gain(DEFAULT_AUDIO_GAIN);
 
     audio_udp_sink = make_udp_sink_f();
 
@@ -197,6 +189,8 @@ void receiver::stop()
  */
 void receiver::set_input_device(const std::string device)
 {
+    std::string error = "";
+
     if (device.empty())
         return;
 
@@ -230,7 +224,17 @@ void receiver::set_input_device(const std::string device)
     }
 
     src.reset();
-    src = osmosdr::source::make(device);
+
+    try
+    {
+        src = osmosdr::source::make(device);
+    }
+    catch (std::runtime_error &x)
+    {
+        error = x.what();
+        src = osmosdr::source::make("file="+get_random_file()+",freq=428e6,rate=96000,repeat=true,throttle=true");
+    }
+
     if(src->get_sample_rate() != 0)
         set_input_rate(src->get_sample_rate());
 
@@ -246,6 +250,11 @@ void receiver::set_input_device(const std::string device)
 
     if (d_running)
         tb->start();
+
+    if (error != "")
+    {
+        throw std::runtime_error(error);
+    }
 }
 
 
@@ -302,7 +311,7 @@ std::vector<std::string> receiver::get_antennas(void) const
     return src->get_antennas();
 }
 
-/** Select antenna conenctor. */
+/** Select antenna connector. */
 void receiver::set_antenna(const std::string &antenna)
 {
     if (!antenna.empty())
@@ -328,7 +337,14 @@ double receiver::set_input_rate(double rate)
             * std::numeric_limits<double>::epsilon());
 
     tb->lock();
-    d_input_rate = src->set_sample_rate(rate);
+    try
+    {
+        d_input_rate = src->set_sample_rate(rate);
+    }
+    catch (std::runtime_error &e)
+    {
+        d_input_rate = 0;
+    }
 
     if (d_input_rate == 0)
     {
@@ -345,10 +361,13 @@ double receiver::set_input_rate(double rate)
         d_input_rate = rate;
     }
 
-    d_quad_rate = d_input_rate / (double)d_decim;
-    dc_corr->set_sample_rate(d_quad_rate);
+    d_decim_rate = d_input_rate / (double)d_decim;
+    d_ddc_decim = std::max(1, (int)(d_decim_rate / TARGET_QUAD_RATE));
+    d_quad_rate = d_decim_rate / d_ddc_decim;
+    dc_corr->set_sample_rate(d_decim_rate);
+    ddc->set_decim_and_samp_rate(d_ddc_decim, d_decim_rate);
     rx->set_quad_rate(d_quad_rate);
-    lo->set_sampling_freq(d_quad_rate);
+    iq_fft->set_quad_rate(d_decim_rate);
     tb->unlock();
 
     return d_input_rate;
@@ -392,17 +411,20 @@ unsigned int receiver::set_input_decim(unsigned int decim)
             d_decim = 1;
         }
 
-        d_quad_rate = d_input_rate / (double)d_decim;
+        d_decim_rate = d_input_rate / (double)d_decim;
     }
     else
     {
-        d_quad_rate = d_input_rate;
+        d_decim_rate = d_input_rate;
     }
 
     // update quadrature rate
-    dc_corr->set_sample_rate(d_quad_rate);
+    d_ddc_decim = std::max(1, (int)(d_decim_rate / TARGET_QUAD_RATE));
+    d_quad_rate = d_decim_rate / d_ddc_decim;
+    dc_corr->set_sample_rate(d_decim_rate);
+    ddc->set_decim_and_samp_rate(d_ddc_decim, d_decim_rate);
     rx->set_quad_rate(d_quad_rate);
-    lo->set_sampling_freq(d_quad_rate);
+    iq_fft->set_quad_rate(d_decim_rate);
 
     if (d_decim >= 2)
     {
@@ -416,7 +438,7 @@ unsigned int receiver::set_input_decim(unsigned int decim)
 
 #ifdef CUSTOM_AIRSPY_KERNELS
     if (input_devstr.find("airspy") != std::string::npos)
-        src->set_bandwidth(d_quad_rate);
+        src->set_bandwidth(d_decim_rate);
 #endif
 
     if (d_running)
@@ -583,7 +605,7 @@ std::vector<std::string> receiver::get_gain_names()
  * @param[out] stop  Upper limit for this gain setting.
  * @param[out] step  The resolution for this gain setting.
  *
- * This function retunrs the range for the requested gain stage.
+ * This function returns the range for the requested gain stage.
  */
 receiver::status receiver::get_gain_range(std::string &name, double *start,
                                           double *stop, double *step) const
@@ -640,7 +662,7 @@ receiver::status receiver::set_auto_gain(bool automatic)
 receiver::status receiver::set_filter_offset(double offset_hz)
 {
     d_filter_offset = offset_hz;
-    lo->set_frequency(-d_filter_offset + d_cw_offset);
+    ddc->set_center_freq(d_filter_offset - d_cw_offset);
 
     return STATUS_OK;
 }
@@ -659,7 +681,7 @@ double receiver::get_filter_offset(void) const
 receiver::status receiver::set_cw_offset(double offset_hz)
 {
     d_cw_offset = offset_hz;
-    lo->set_frequency(-d_filter_offset + d_cw_offset);
+    ddc->set_center_freq(d_filter_offset - d_cw_offset);
     rx->set_cw_offset(d_cw_offset);
 
     return STATUS_OK;
@@ -725,6 +747,11 @@ void receiver::set_iq_fft_size(int newsize)
     iq_fft->set_fft_size(newsize);
 }
 
+void receiver::set_iq_fft_window(int window_type)
+{
+    iq_fft->set_window_type(window_type);
+}
+
 /** Get latest baseband FFT data. */
 void receiver::get_iq_fft_data(std::complex<float>* fftPoints, unsigned int &fftsize)
 {
@@ -973,19 +1000,24 @@ receiver::status receiver::start_audio_recording(const std::string filename)
         return STATUS_ERROR;
     }
 
-    // not strictly necessary to lock but I think it is safer
-    tb->lock();
-
     // if this fails, we don't want to go and crash now, do we
     try {
-        wav_sink->open(filename.c_str());
-        wav_sink->set_sample_rate((unsigned int) d_audio_rate);
+#if GNURADIO_VERSION < 0x030900
+        wav_sink = gr::blocks::wavfile_sink::make(filename.c_str(), 2,
+                                                  (unsigned int) d_audio_rate,
+                                                  16);
+#else
+        wav_sink = gr::blocks::wavfile_sink::make(filename.c_str(), 2,
+                                                  (unsigned int) d_audio_rate,
+                                                  gr::blocks::FORMAT_WAV, gr::blocks::FORMAT_PCM_16);
+#endif
     }
     catch (std::runtime_error &e) {
         std::cout << "Error opening " << filename << ": " << e.what() << std::endl;
         return STATUS_ERROR;
     }
 
+    tb->lock();
     tb->connect(rx, 0, wav_sink, 0);
     tb->connect(rx, 1, wav_sink, 1);
     tb->unlock();
@@ -1019,6 +1051,7 @@ receiver::status receiver::stop_audio_recording()
     tb->disconnect(rx, 0, wav_sink, 0);
     tb->disconnect(rx, 1, wav_sink, 1);
     tb->unlock();
+    wav_sink.reset();
     d_recording_wav = false;
 
     std::cout << "Audio recorder stopped" << std::endl;
@@ -1071,12 +1104,14 @@ receiver::status receiver::start_audio_playback(const std::string filename)
     tb->disconnect(rx, 1, audio_gain1, 0);
     tb->disconnect(rx, 0, audio_fft, 0);
     tb->disconnect(rx, 0, audio_udp_sink, 0);
+    tb->disconnect(rx, 1, audio_udp_sink, 1);
     tb->connect(rx, 0, audio_null_sink0, 0); /** FIXME: other channel? */
     tb->connect(rx, 1, audio_null_sink1, 0); /** FIXME: other channel? */
     tb->connect(wav_src, 0, audio_gain0, 0);
     tb->connect(wav_src, 1, audio_gain1, 0);
     tb->connect(wav_src, 0, audio_fft, 0);
     tb->connect(wav_src, 0, audio_udp_sink, 0);
+    tb->connect(wav_src, 1, audio_udp_sink, 1);
     start();
 
     std::cout << "Playing audio from " << filename << std::endl;
@@ -1093,12 +1128,14 @@ receiver::status receiver::stop_audio_playback()
     tb->disconnect(wav_src, 1, audio_gain1, 0);
     tb->disconnect(wav_src, 0, audio_fft, 0);
     tb->disconnect(wav_src, 0, audio_udp_sink, 0);
+    tb->disconnect(wav_src, 1, audio_udp_sink, 1);
     tb->disconnect(rx, 0, audio_null_sink0, 0);
     tb->disconnect(rx, 1, audio_null_sink1, 0);
     tb->connect(rx, 0, audio_gain0, 0);
     tb->connect(rx, 1, audio_gain1, 0);
     tb->connect(rx, 0, audio_fft, 0);  /** FIXME: other channel? */
     tb->connect(rx, 0, audio_udp_sink, 0);
+    tb->connect(rx, 1, audio_udp_sink, 1);
     start();
 
     /* delete wav_src since we can not change file name */
@@ -1108,9 +1145,9 @@ receiver::status receiver::stop_audio_playback()
 }
 
 /** Start UDP streaming of audio. */
-receiver::status receiver::start_udp_streaming(const std::string host, int port)
+receiver::status receiver::start_udp_streaming(const std::string host, int port, bool stereo)
 {
-    audio_udp_sink->start_streaming(host, port);
+    audio_udp_sink->start_streaming(host, port, stereo);
     return STATUS_OK;
 }
 
@@ -1134,29 +1171,24 @@ receiver::status receiver::start_iq_recording(const std::string filename)
         return STATUS_ERROR;
     }
 
-    // iq_sink was created in the constructor
-    if (iq_sink) {
-        tb->lock();
-        if (!iq_sink->open(filename.c_str()))
-        {
-            status = STATUS_ERROR;
-        }
-        else
-        {
-            if (d_decim >= 2)
-                tb->connect(input_decim, 0, iq_sink, 0);
-            else
-                tb->connect(src, 0, iq_sink, 0);
-
-            d_recording_iq = true;
-        }
-        tb->unlock();
+    try
+    {
+        iq_sink = gr::blocks::file_sink::make(sizeof(gr_complex), filename.c_str(), true);
     }
-    else {
-        std::cout << __func__ << ": I/Q file sink does not exist" << std::endl;
+    catch (std::runtime_error &e)
+    {
+        std::cout << __func__ << ": couldn't open I/Q file" << std::endl;
         return STATUS_ERROR;
     }
 
+    tb->lock();
+    if (d_decim >= 2)
+        tb->connect(input_decim, 0, iq_sink, 0);
+    else
+        tb->connect(src, 0, iq_sink, 0);
+    d_recording_iq = true;
+    tb->unlock();
+
     return status;
 }
 
@@ -1177,6 +1209,7 @@ receiver::status receiver::stop_iq_recording()
         tb->disconnect(src, 0, iq_sink, 0);
 
     tb->unlock();
+    iq_sink.reset();
     d_recording_iq = false;
 
     return STATUS_OK;
@@ -1260,63 +1293,45 @@ void receiver::get_sniffer_data(float * outbuff, unsigned int &num)
 /** Convenience function to connect all blocks. */
 void receiver::connect_all(rx_chain type)
 {
+    gr::basic_block_sptr b;
+
+    // Setup source
+    b = src;
+
+    // Pre-processing
+    if (d_decim >= 2)
+    {
+        tb->connect(b, 0, input_decim, 0);
+        b = input_decim;
+    }
+
+    if (d_recording_iq)
+    {
+        // We record IQ with minimal pre-processing
+        tb->connect(b, 0, iq_sink, 0);
+    }
+
+    tb->connect(b, 0, iq_swap, 0);
+    b = iq_swap;
+
+    if (d_dc_cancel)
+    {
+        tb->connect(b, 0, dc_corr, 0);
+        b = dc_corr;
+    }
+
+    // Visualization
+    tb->connect(b, 0, iq_fft, 0);
+
+    // RX demod chain
     switch (type)
     {
-    case RX_CHAIN_NONE:
-        if (d_decim >= 2)
-        {
-            tb->connect(src, 0, input_decim, 0);
-            tb->connect(input_decim, 0, iq_swap, 0);
-        }
-        else
-        {
-            tb->connect(src, 0, iq_swap, 0);
-        }
-        if (d_dc_cancel)
-        {
-            tb->connect(iq_swap, 0, dc_corr, 0);
-            tb->connect(dc_corr, 0, iq_fft, 0);
-        }
-        else
-        {
-            tb->connect(iq_swap, 0, iq_fft, 0);
-        }
-        break;
-
     case RX_CHAIN_NBRX:
         if (rx->name() != "NBRX")
         {
             rx.reset();
             rx = make_nbrx(d_quad_rate, d_audio_rate);
         }
-        if (d_decim >= 2)
-        {
-            tb->connect(src, 0, input_decim, 0);
-            tb->connect(input_decim, 0, iq_swap, 0);
-        }
-        else
-        {
-            tb->connect(src, 0, iq_swap, 0);
-        }
-        if (d_dc_cancel)
-        {
-            tb->connect(iq_swap, 0, dc_corr, 0);
-            tb->connect(dc_corr, 0, iq_fft, 0);
-            tb->connect(dc_corr, 0, mixer, 0);
-        }
-        else
-        {
-            tb->connect(iq_swap, 0, iq_fft, 0);
-            tb->connect(iq_swap, 0, mixer, 0);
-        }
-        tb->connect(lo, 0, mixer, 1);
-        tb->connect(mixer, 0, rx, 0);
-        tb->connect(rx, 0, audio_fft, 0);
-        tb->connect(rx, 0, audio_udp_sink, 0);
-        tb->connect(rx, 0, audio_gain0, 0);
-        tb->connect(rx, 1, audio_gain1, 0);
-        tb->connect(audio_gain0, 0, audio_snk, 0);
-        tb->connect(audio_gain1, 0, audio_snk, 1);
         break;
 
     case RX_CHAIN_WFMRX:
@@ -1325,49 +1340,27 @@ void receiver::connect_all(rx_chain type)
             rx.reset();
             rx = make_wfmrx(d_quad_rate, d_audio_rate);
         }
-        if (d_decim >= 2)
-        {
-            tb->connect(src, 0, input_decim, 0);
-            tb->connect(input_decim, 0, iq_swap, 0);
-        }
-        else
-        {
-            tb->connect(src, 0, iq_swap, 0);
-        }
-        if (d_dc_cancel)
-        {
-            tb->connect(iq_swap, 0, dc_corr, 0);
-            tb->connect(dc_corr, 0, iq_fft, 0);
-            tb->connect(dc_corr, 0, mixer, 0);
-        }
-        else
-        {
-            tb->connect(iq_swap, 0, iq_fft, 0);
-            tb->connect(iq_swap, 0, mixer, 0);
-        }
-        tb->connect(lo, 0, mixer, 1);
-        tb->connect(mixer, 0, rx, 0);
-        tb->connect(rx, 0, audio_fft, 0);
-        tb->connect(rx, 0, audio_udp_sink, 0);
-        tb->connect(rx, 0, audio_gain0, 0);
-        tb->connect(rx, 1, audio_gain1, 0);
-        tb->connect(audio_gain0, 0, audio_snk, 0);
-        tb->connect(audio_gain1, 0, audio_snk, 1);
         break;
 
     default:
         break;
     }
 
-    // reconnect recorders and sniffers
-    if (d_recording_iq)
+    // Audio path (if there is a receiver)
+    if (type != RX_CHAIN_NONE)
     {
-        if (d_decim >= 2)
-            tb->connect(input_decim, 0, iq_sink, 0);
-        else
-            tb->connect(src, 0, iq_sink, 0);
+        tb->connect(b, 0, ddc, 0);
+        tb->connect(ddc, 0, rx, 0);
+        tb->connect(rx, 0, audio_fft, 0);
+        tb->connect(rx, 0, audio_udp_sink, 0);
+        tb->connect(rx, 1, audio_udp_sink, 1);
+        tb->connect(rx, 0, audio_gain0, 0);
+        tb->connect(rx, 1, audio_gain1, 0);
+        tb->connect(audio_gain0, 0, audio_snk, 0);
+        tb->connect(audio_gain1, 0, audio_snk, 1);
     }
 
+    // Recorders and sniffers
     if (d_recording_wav)
     {
         tb->connect(rx, 0, wav_sink, 0);
