@@ -79,8 +79,11 @@ void RemoteControl::start_server()
 /*! \brief Stop the server. */
 void RemoteControl::stop_server()
 {
-    if (rc_socket != 0)
+    if (rc_socket != 0) {
         rc_socket->close();
+        rc_socket->deleteLater();
+        rc_socket = 0;
+    }
 
     if (rc_server.isListening())
         rc_server.close();
@@ -162,6 +165,11 @@ void RemoteControl::setHosts(QStringList hosts)
  */
 void RemoteControl::acceptConnection()
 {
+    if (rc_socket)
+    {
+        rc_socket->close();
+        rc_socket->deleteLater();
+    }
     rc_socket = rc_server.nextPendingConnection();
 
     // check if host is allowed
@@ -171,6 +179,8 @@ void RemoteControl::acceptConnection()
         std::cout << "*** Remote connection attempt from " << address.toStdString()
                   << " (not in allowed list)" << std::endl;
         rc_socket->close();
+        rc_socket->deleteLater();
+        rc_socket = 0;
     }
     else
     {
@@ -237,6 +247,8 @@ void RemoteControl::startRead()
     {
         // FIXME: for now we assume 'close' command
         rc_socket->close();
+        rc_socket->deleteLater();
+        rc_socket = 0;
         return;
     }
     else
@@ -252,7 +264,7 @@ void RemoteControl::startRead()
 /*! \brief Slot called when the receiver is tuned to a new frequency.
  *  \param freq The new frequency in Hz.
  *
- * Note that this is the frequency gqrx is receiveing on, i.e. the
+ * Note that this is the frequency gqrx is receiving on, i.e. the
  * hardware frequency + the filter offset.
  */
 void RemoteControl::setNewFrequency(qint64 freq)
@@ -355,6 +367,29 @@ void RemoteControl::setReceiverStatus(bool enabled)
     receiver_running = enabled;
 }
 
+/*! \brief Set available gain settings (from mainwindow). */
+void RemoteControl::setGainStages(gain_list_t &gain_list)
+{
+    gains = gain_list;
+}
+
+/*! \brief Set value for a specific gain setting (from DockInputCtl). */
+bool RemoteControl::setGain(QString name, double gain)
+{
+    for(auto &g : gains)
+    {
+        if(name == QString::fromStdString(g.name))
+        {
+            if(gain != g.value) {
+                g.value = gain;
+                emit gainChanged(name, gain);
+            }
+            return true;
+        }
+    }
+    return false;
+}
+
 
 /*! \brief Convert mode string to enum (DockRxOpt::rxopt_mode_idx)
  *  \param mode The Hamlib rigctld compatible mode string
@@ -559,13 +594,37 @@ QString RemoteControl::cmd_get_level(QStringList cmdlist)
     QString lvl = cmdlist.value(1, "");
 
     if (lvl == "?")
-       answer = QString("SQL STRENGTH\n");
+    {
+        QStringList names;
+        for(auto &g : gains)
+            names.push_back(QString("%1_GAIN").arg(QString::fromStdString(g.name)));
+        answer = QString("SQL STRENGTH %1\n").arg(names.join(" "));
+    }
     else if (lvl.compare("STRENGTH", Qt::CaseInsensitive) == 0 || lvl.isEmpty())
-       answer = QString("%1\n").arg(signal_level, 0, 'f', 1);
+    {
+        answer = QString("%1\n").arg(signal_level, 0, 'f', 1);
+    }
     else if (lvl.compare("SQL", Qt::CaseInsensitive) == 0)
-       answer = QString("%1\n").arg(squelch_level, 0, 'f', 1);
+    {
+        answer = QString("%1\n").arg(squelch_level, 0, 'f', 1);
+    }
+    else if (lvl.contains(QRegExp("_GAIN$")))
+    {
+        QString name = lvl.remove(QRegExp("_GAIN$"));
+        answer = QString("RPRT 1\n");
+        for(auto &g : gains)
+        {
+            if(name == QString::fromStdString(g.name))
+            {
+                answer = QString("%1\n").arg(g.value);
+                break;
+            }
+        }
+    }
     else
-       answer = QString("RPRT 1\n");
+    {
+        answer = QString("RPRT 1\n");
+    }
 
     return answer;
 }
@@ -577,7 +636,12 @@ QString RemoteControl::cmd_set_level(QStringList cmdlist)
     QString lvl = cmdlist.value(1, "");
 
     if (lvl == "?")
-        answer = QString("SQL\n");
+    {
+        QStringList names;
+        for(auto &g : gains)
+            names.push_back(QString("%1_GAIN").arg(QString::fromStdString(g.name)));
+        answer = QString("SQL %1\n").arg(names.join(" "));
+    }
     else if (lvl.compare("SQL", Qt::CaseInsensitive) == 0)
     {
         bool ok;
@@ -593,6 +657,17 @@ QString RemoteControl::cmd_set_level(QStringList cmdlist)
             answer = QString("RPRT 1\n");
         }
     }
+    else if (lvl.contains(QRegExp("_GAIN$")))
+    {
+        QString name = lvl.remove(QRegExp("_GAIN$"));
+
+        bool ok;
+        double gain = cmdlist.value(2, "ERR").toDouble(&ok);
+        if (ok && setGain(name, gain))
+            answer = QString("RPRT 0\n");
+        else
+            answer = QString("RPRT 1\n");
+    }
     else
     {
         answer = QString("RPRT 1\n");
